--
-- ChangeBodyType for Broughan
--
-- author  Mythos
-- date  23/05/2016
-- www.ARM-team.gr
-- Copyright (C) ARM Team, All Rights Reserved.

BBodyType = {};

function BBodyType.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Fillable, specializations);
end;

--________________________________________________________________________________________________________________________________________________________
function BBodyType:load(xmlFile)
	-- Functions
	self.MPCode = SpecializationUtil.callSpecializationsFunction("MPCode");
	self.registerFillVolume = SpecializationUtil.callSpecializationsFunction("registerFillVolume");
	self.registerFillTypes = SpecializationUtil.callSpecializationsFunction("registerFillTypes");
	
	-- Indexes
	self.backDischarge = Utils.indexToObject(self.components,getXMLString(xmlFile,"vehicle.backDischarge#index"));
	self.strobe = Utils.indexToObject(self.components,getXMLString(xmlFile,"vehicle.strobe#index"));

	-- Booleans & Vars
	self.changeBody = 2;
	
	self.strobeOn = true;
	self.strobeT1 = 250;
	self.strobeT2 = 125;
	self.counter = 0;
	self.safe = false;
	self.safe2 = false;
	
	-- Body Parts
	self.grainPart = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.grainBody#part1"));
	self.silagePart1 = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.silageBody#part1"));
	self.silagePart2 = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.silageBody#part2"));
	
	-- Register fillVolume and capacity
	self.grainIndex = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.grainBody#index"));
	self.silageIndex = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.silageBody#index"));
	
	self.grainCapacity = getXMLFloat(xmlFile, "vehicle.grainBody#capacity");
	self.silageCapacity = getXMLFloat(xmlFile, "vehicle.silageBody#capacity");
	
	self.capacity = self.silageCapacity;
	self:registerFillVolume(self.silageIndex, self.capacity);

	-- Register fillTypes
	self.allFillTypes = getXMLString(xmlFile, "vehicle.allFillTypes#fillTypes");
	self.grainFillTypes = getXMLString(xmlFile, "vehicle.grainBody#fillTypes");
	self.silageFillTypes = getXMLString(xmlFile, "vehicle.silageBody#fillTypes");
	
	self:registerFillTypes(self.silageFillTypes);

end;

--________________________________________________________________________________________________________________________________________________________
function BBodyType:registerFillVolume(index, capacity)
	local num = table.getn(self.fillVolumes);
	for i=1, num do
		self.fillVolumes[i] = nil;
	end;
	
	local fillVolume = {};
	fillVolume.baseNode = index;
	fillVolume.volume = createFillPlaneShape(fillVolume.baseNode, "fillPlane", capacity, 1, math.rad(35), math.rad(35), 0.9, true);
	fillVolume.scrollSpeedDischarge = { Utils.getVectorFromString("0 0 0") };
	fillVolume.scrollSpeedLoad = { Utils.getVectorFromString("0 0 0") };
	for i=1,3 do
		fillVolume.scrollSpeedDischarge[i] = fillVolume.scrollSpeedDischarge[i] / 1000;
		fillVolume.scrollSpeedLoad[i] = fillVolume.scrollSpeedLoad[i] / 1000;
	end;
	fillVolume.uvPosition = {0, 0, 0};
	if fillVolume.volume ~= nil and fillVolume.volume ~= 0 then
		link(fillVolume.baseNode, fillVolume.volume);
		table.insert(self.fillVolumes, fillVolume);
	end;
end;

--________________________________________________________________________________________________________________________________________________________
function BBodyType:registerFillTypes(fillTypes)
	
	local types = Utils.splitString(" ", self.allFillTypes);
	for k,v in pairs(types) do
		local allfillType = Fillable.fillTypeNameToInt[v];
		if allfillType ~= nil then
			self.fillTypes[allfillType] = nil;
		end;
	end;

	if fillTypes ~= nil then
		local types = Utils.splitString(" ", fillTypes);
		for k,v in pairs(types) do
			local fillType = Fillable.fillTypeNameToInt[v];
			if fillType ~= nil then
				self.fillTypes[fillType] = true;
			else
				print("Warning: '"..self.configFileName.. "' has invalid fillType '"..v.."'.");
			end;
		end;
	end;
end;

--________________________________________________________________________________________________________________________________________________________
function BBodyType:delete()
end;

--________________________________________________________________________________________________________________________________________________________
function BBodyType:mouseEvent(posX, posY, isDown, isUp, button)
end;

--________________________________________________________________________________________________________________________________________________________
function BBodyType:keyEvent(unicode, sym, modifier, isDown)
end;

--________________________________________________________________________________________________________________________________________________________
function BBodyType:update(dt)
	
	if self:getIsActiveForInput() and self:getIsActive() then
		
		if self.fillLevel == 0 and self.tipState == 0 then
			if InputBinding.hasEvent(InputBinding.BROUGHAN_BODY_MODE) then
				self.changeBody = self.changeBody + 1;
			
				if self.changeBody == 1  then
					self:MPCode(1);
				elseif self.changeBody == 2  then
					self:MPCode(2);
				elseif self.changeBody == 3  then
					self:MPCode(1);
				end;
			end;
		end;
		
	end;
	
end;

--________________________________________________________________________________________________________________________________________________________
function BBodyType:updateTick(dt)

	if self.attacherVehicle ~= nil then
		if (self.attacherVehicle:getIsActiveForInput(false) and self.attacherVehicle.beaconLightsActive) or (g_gui.currentGuiName == "ChatDialog" and self.attacherVehicle.beaconLightsActive) then
			self.safe2 = false;
			if self.counter < 2 then
				if not self.safe then
					self:MPCode(3, self.strobeOn);
					self.safe = true;
				end;
				
				self.strobeT1 = self.strobeT1 - dt;
				if self.strobeT1 < 0 then
					self.strobeOn = not self.strobeOn;
					self.counter = self.counter + 1;
					self.safe = false;
					self.strobeT1 = 250;
				end;
			elseif self.counter >= 2 then
				if not self.safe then
					self:MPCode(3, self.strobeOn);
					self.safe = true;
				end;
				
				self.strobeT2 = self.strobeT2 - dt;
				if self.strobeT2 < 0 then
					self.strobeOn = not self.strobeOn;
					self.counter = self.counter + 1;
					self.safe = false;
					self.strobeT2 = 125;
					if self.counter > 6 then
						self.counter = 0;
					end;
				end;
			end;
		else
			if not self.safe2 then
				self:MPCode(3, false);
				self.safe2 = true;
			end;
			self.strobeOn = true;
			self.strobeT1 = 250;
			self.strobeT2 = 125;
			self.counter = 0;
			self.safe = false;
		end;
	end;
	
end;

--________________________________________________________________________________________________________________________________________________________
function BBodyType:onDetach()
	self:MPCode(3, false);
	self.strobeOn = true;
	self.strobeT1 = 250;
	self.strobeT2 = 125;
	self.counter = 0;
	self.safe = false;
end;

--________________________________________________________________________________________________________________________________________________________
function BBodyType:MPCode(mode, state, a1, noEventSend)
	BBodyTypeEvent.sendEvent(self, mode, state, a1, noEventSend);
	--______________________________________________________________________________
	if mode == 1 then
	--_________________________________ GRAIN BODY __________________________________
		self.changeBody = 1;
		self.capacity = self.grainCapacity;
		self:registerFillVolume(self.grainIndex, self.capacity);
		self:registerFillTypes(self.grainFillTypes);
		
		setTranslation(self.backDischarge, 0, 3, 2.5);
		
		setVisibility(self.grainPart, true);
		setVisibility(self.silagePart1, false);
		setVisibility(self.silagePart2, false);
	--______________________________________________________________________________
	elseif mode == 2 then
	--_________________________________ SILAGE BODY__________________________________
		self.changeBody = 2;
		self.capacity = self.silageCapacity;
		self:registerFillVolume(self.silageIndex, self.capacity);
		self:registerFillTypes(self.silageFillTypes);
		
		setTranslation(self.backDischarge, 0, 3, -4);
		
		setVisibility(self.grainPart, false);
		setVisibility(self.silagePart1, true);
		setVisibility(self.silagePart2, true);
	--______________________________________________________________________________
	elseif mode == 3 then
	--____________________________________ STROBE MODE _____________________________
		if state then
			setVisibility(self.strobe, true);
		else
			setVisibility(self.strobe, false);
		end;
	end;

end;

--________________________________________________________________________________________________________________________________________________________
function BBodyType:draw()	
	
	if self.fillLevel == 0 and self.tipState == 0 then
		if self.changeBody == 1  then
			g_currentMission:addHelpButtonText(string.format(g_i18n:getText("BROUGHAN_SILAGE_BODY"), self.typeDesc), InputBinding.BROUGHAN_BODY_MODE);
		elseif self.changeBody == 2  then
			g_currentMission:addHelpButtonText(string.format(g_i18n:getText("BROUGHAN_GRAIN_BODY"), self.typeDesc), InputBinding.BROUGHAN_BODY_MODE);
		end;
	end;

end;

--________________________________________________________________________________________________________________________________________________________
function BBodyType:loadFromAttributesAndNodes(xmlFile, key, resetVehicles)
	if not resetVehicles then
		self.changeBody = Utils.getNoNil(getXMLInt(xmlFile, key.."#changeBody"),2);
		
		if self.changeBody ~=nil and self.changeBody ~=2 then
			self:MPCode(self.changeBody);
		end;
		
		local fillLevel = getXMLFloat(xmlFile, key.."#fillLevel");
		local fillType = getXMLString(xmlFile, key.."#fillType");
		if fillLevel ~= nil and fillType ~= nil then
			local fillTypeInt = Fillable.fillTypeNameToInt[fillType];
			if fillTypeInt ~= nil then
				if self.changeBody ~=2 then
					local fillSourceStruct = {x=0,y=0,z=0, d1x=0,d1y=0,d1z=0, d2x=0,d2y=0,d2z=0};
				end;
				self:setFillLevel(fillLevel, fillTypeInt, false, fillSourceStruct);
			end;
		end;
	
	end;
	
	return BaseMission.VEHICLE_LOAD_OK;
end;

--________________________________________________________________________________________________________________________________________________________
function BBodyType:getSaveAttributesAndNodes(nodeIdent)
	local attributes = 'changeBody="'..tostring(self.changeBody)..'"';

	local nodes = nil;
	
	return attributes,nodes;
end;

--________________________________________________________________________________________________________________________________________________________
function BBodyType:readStream(streamId, connection)
	self.changeBody = streamReadInt32(streamId);
	if self.changeBody ~=nil and self.changeBody ~=1 then
		self:MPCode(self.changeBody, nil, nil, true);
	end;
	
	local fillLevel = streamReadFloat32(streamId);
	local fillType = streamReadUIntN(streamId, Fillable.sendNumBits);
	if self.changeBody ~=1 then
		local fillSourceStruct = {x=0,y=0,z=0, d1x=0,d1y=0,d1z=0, d2x=0,d2y=0,d2z=0};
	end;
	self:setFillLevel(fillLevel, fillType, false, fillSourceStruct);
end;

--________________________________________________________________________________________________________________________________________________________
function BBodyType:writeStream(streamId, connection)
	streamWriteInt32(streamId, self.changeBody);
	streamWriteFloat32(streamId, self.fillLevel);
	streamWriteUIntN(streamId, self.currentFillType, Fillable.sendNumBits);
end;

--**********************************************************************************************************************************************************************************
BBodyTypeEvent = {};
BBodyTypeEvent_mt = Class(BBodyTypeEvent, Event);

InitEventClass(BBodyTypeEvent, "BBodyTypeEvent");

function BBodyTypeEvent:emptyNew()
    local self = Event:new(BBodyTypeEvent_mt);
    self.className="BBodyTypeEvent";
    return self;
end;

function BBodyTypeEvent:new(vehicle, mode, state, a1)
    local self = BBodyTypeEvent:emptyNew()
    self.vehicle = vehicle;
	self.mode = mode;
	self.state = state;
	self.a1 = a1;
    return self;
end;

function BBodyTypeEvent:readStream(streamId, connection)
    local id = streamReadInt32(streamId);
	self.mode = streamReadInt32(streamId);
	self.state = streamReadBool(streamId);
	self.a1 = streamReadInt32(streamId);
    self.vehicle = networkGetObject(id);
    self:run(connection);
end;

function BBodyTypeEvent:writeStream(streamId, connection)
    streamWriteInt32(streamId, networkGetObjectId(self.vehicle));
	streamWriteInt32(streamId, self.mode);
	streamWriteBool(streamId, self.state);
	streamWriteInt32(streamId, self.a1);
end;

function BBodyTypeEvent:run(connection)
	self.vehicle:MPCode(self.mode, self.state, self.a1, true);
    if not connection:getIsServer() then
        g_server:broadcastEvent(BBodyTypeEvent:new(self.vehicle, self.mode, self.state, self.a1), nil, connection, self.vehicle);
    end;
end;

function BBodyTypeEvent.sendEvent(vehicle, mode, state, a1, noEventSend)
	if noEventSend == nil or noEventSend == false then
		if g_server ~= nil then
			g_server:broadcastEvent(BBodyTypeEvent:new(vehicle, mode, state, a1), nil, nil, vehicle);
		else
			g_client:getServerConnection():sendEvent(BBodyTypeEvent:new(vehicle, mode, state, a1));
		end;
	end;
end;